/*
 * DebugLevel class
 *
 * Author: M. Heidi McClure
 * Date Started: approx. 2/10/97
 * Date Last Revised: 4/12/97
 */

import java.util.Properties;

/**
 * This class handles the debug print statements.  One or more
 * of the DEBUG variables may be passed in from the call to the java
 * virtual machine with the -D option, information will be printed
 * to the Console window and to the window from which UTMOST2 was
 * started.  That is, to call for FLOW_DEBUGGING, start UTMOST2 as
 * follows:
 * <p>
 * java -DDEBUG_FLOW UTMOST2
 * <p>
 *
 * @author      M. Heidi McClure
 * @version UTMOST2 1.0
 * @see UTMOST2
 * @see Console
 */

public class DebugLevel {

    private static int level = 0;

    /**
     * This is the default... no corresponding -D property.
     */
    public static final int NO_DEBUGGING =    0;
    /**
     * Show timing information by using -DDEBUG_TIMING
     * <p>
     * Currently this doesn't work.
     */
    public static final int TIMING_DEBUGGING =    1;
    /**
     * Show traces by using -DDEBUG_TRACE
     * <p>
     * Currently this doesn't work.
     */
    public static final int TRACE_DEBUGGING =    2;
    /**
     * Show sector debugging information by using -DDEBUG_SECTOR
     */
    public static final int SECTOR_DEBUGGING  =    4;
    /**
     * Show flow debugging information by using -DDEBUG_FLOW
     */
    public static final int FLOW_DEBUGGING =    8;
    /**
     * Show user movement debugging information by using -DDEBUG_MOVE
     */
    public static final int MOVE_DEBUGGING =    16;
    /**
     * Show segment calculations debugging information by using -DDEBUG_SEGMENT
     */
    public static final int SEGMENT_DEBUGGING =    32;
    /**
     * Show intersection calculation debugging information by using -DDEBUG_INTERSECT
     */
    public static final int INTERSECT_DEBUGGING =    64;
    /**
     * Show misc. user debugging information by using -DDEBUG_USER
     */
    public static final int USER_DEBUGGING =    128;
    /**
     * Show visible sector calculation debugging information by using -DDEBUG_VISIBLE
     */
    public static final int VISIBLE_DEBUGGING =    256;
    /**
     * Show inside sector calculation debugging information by using -DDEBUG_INSIDE
     */
    public static final int INSIDE_DEBUGGING =    512;

    /**
     * Use the original Visible Sector calculation algorithm by
     * using -DDEBUG_NO_VIS_OPT  (default is to use the better algorithm)
     */
    public static final int NO_VIS_OPT_DEBUGGING =    1024;

    /**
     * Use the original Boundary Cross Algorithms by using
     * -DDEBUG_NO_ADJ_OPT  (default is to use the better algorithm)
     */
    public static final int NO_ADJ_OPT_DEBUGGING =    2048;

    /**
     * Flag that simulator connection is required
     * -DSIMULATOR
     */
    public static final int SIMULATOR =    4096;

    /**
     * Flag that POCAT is the simulator to use.
     * -DPOCAT
     */
    public static final int POCAT =    8192;

    /**
     * Flag that RACESIM is the simulator to user.
     * -DRACESIM
     */
    public static final int RACESIM =    16384;

    /**
     *  If DEBUG_TRACE is enabled a stack trace is printed.
     */
    public static void debugTrace() {
        parseDebugProperties();
    if (debugging(TRACE_DEBUGGING)) {

        try {
        throw new Exception("Tracing Stack");
        } catch (Exception exc) {
        System.out.println("\n--------------------------------------");
        exc.printStackTrace();
        System.out.println("--------------------------------------\n");
        }
    }

    }

    /**
     *  Always prints out a stack trace.
     */
    public static void debugTraceAlways() {
    try {
        throw new Exception("Tracing Stack");
    } catch (Exception exc) {
        System.out.println("\n--------------------------------------");
        exc.printStackTrace();
        System.out.println("--------------------------------------\n");
    }
    }

    /**
     * Turns on this debugging level.
     */

    public static void turnOn(int aDebugLevel) {
        level |= aDebugLevel;
    }

    /**
     * Turns on all debugging.
     */

    public static void turnOnAll() {
        level = ~0; // i.e. complement of 0
    }

    /**
     * Turns off this debugging level.
     */

    public static void turnOff(int aDebugLevel) {
        level &= ~aDebugLevel;
    }

    /**
     * Turns off all debugging.
     */

    public static void turnOffAll() {
        level = 0;
    }

    /**
     * Returns true if debugging is turned on for this level.
     */

    public static boolean debugging(int aDebugLevel) {
        return ((level & aDebugLevel) != 0);
    }

    public static double getMaxWatts() {
        double max_watts = 0;
        Properties p = System.getProperties();

        if (p.getProperty("WATTS") == null) {
            return 1.2;
        } else {
            return new Double(p.getProperty("WATTS")).doubleValue();
        }
    }

    /**
     * Prints a message and a time stamp for timing debugging.
     */

    public static void printTime(String message) {
        System.out.println("time " + System.currentTimeMillis() +
            " : " + message);
    }

    /**
     * Parses standard Debug property names.
     */

    public static void parseDebugProperties() {

        Properties p = System.getProperties();

    if ((p.getProperty("DEBUG_ALL") != null) ||
                (p.getProperty("DEBUG") != null)) {
        turnOnAll();
    } else {
        if (p.getProperty("DEBUG_TIMING") != null) {
            turnOn(TIMING_DEBUGGING);
        }
        if (p.getProperty("DEBUG_TRACE") != null) {
            turnOn(TRACE_DEBUGGING);
        }
        if (p.getProperty("DEBUG_SECTOR") != null) {
            turnOn(SECTOR_DEBUGGING);
        }
        if (p.getProperty("DEBUG_FLOW") != null) {
            turnOn(FLOW_DEBUGGING);
        }
        if (p.getProperty("DEBUG_MOVE") != null) {
            turnOn(MOVE_DEBUGGING);
        }
        if (p.getProperty("DEBUG_SEGMENT") != null) {
            turnOn(SEGMENT_DEBUGGING);
        }
        if (p.getProperty("DEBUG_INTERSECT") != null) {
            turnOn(INTERSECT_DEBUGGING);
        }
        if (p.getProperty("DEBUG_USER") != null) {
            turnOn(USER_DEBUGGING);
        }
        if (p.getProperty("DEBUG_VISIBLE") != null) {
            turnOn(VISIBLE_DEBUGGING);
        }
        if (p.getProperty("DEBUG_INSIDE") != null) {
            turnOn(INSIDE_DEBUGGING);
        }
        if (p.getProperty("DEBUG_NO_VIS_OPT") != null) {
            turnOn(NO_VIS_OPT_DEBUGGING);
        }
        if (p.getProperty("DEBUG_NO_ADJ_OPT") != null) {
            turnOn(NO_ADJ_OPT_DEBUGGING);
        }
        if (p.getProperty("SIMULATOR") != null) {
            turnOn(SIMULATOR);
            if (p.getProperty("POCAT") != null) {
                turnOn(POCAT);
            } else if (p.getProperty("RACESIM") != null) {
                turnOn(RACESIM);
            }
        }
    }
    }
}
