/*
 * LineSegment class
 *
 * Author: M. Heidi McClure
 * Date Started: 1/16/97
 * Date Last Revised: 4/12/97
 */

/**
 * This class is a line segment definition class.  That is, the class
 * represents a linear line segment with a starting (x,y) and an
 * ending (x,y).  Based on these coordinates, the mx + b = y information
 * is calculated (m is the slope and b is the y intercept).
 * <p>
 * This class also contains a few very key methods used by the UTMOST2
 * model.  Perhaps most important is the intersects() method.  This
 * method will set a User's boundary cross (x,y) coordinates to the
 * intersection of this line segment with a passed in line segment,
 * if there is an intersection.  If there is no intersection, the
 * user's values boundary cross (x,y) are not changed.
 * <p>
 * This intersect method is used by UserStats' writeBoundaryResults method
 *
 * @author      M. Heidi McClure
 * @version UTMOST2 1.0
 * @see User
 * @see UTMOST2
 * @see UserStats#writeBoundaryResults
 */
public class LineSegment {

    private int name;

    private double x1;
    private double y1;

    private double x2;
    private double y2;

    private double m;
    private double b;

    private boolean vertical;

    private int [] sectorList;
    /**
     * Constructor which will create a LineSegment object.
     * <p>
     * It uses the input (x1,y1) and (x2,y2) coordinates to calculate
     * values for m and b corresponding to mx+b=y equation defined by
     * the coordinates.
     *
     * @param x_1 is the x value for the first point of line segment
     * @param y_1 is the y value for the first point of line segment
     * @param x_2 is the x value for the second point of line segment
     * @param y_2 is the y value for the second point of line segment
     */
    public LineSegment(double x_1, double y_1, double x_2, double y_2) {
        new LineSegment(-1, x_1, y_1, x_2, y_2, -1);
    }
    /**
     * Constructor which will create a LineSegment object.
     * <p>
     * It uses the input (x1,y1) and (x2,y2) coordinates to calculate
     * values for m and b corresponding to mx+b=y equation defined by
     * the coordinates.
     * <p>
     * This line segment may have a name (an index into an array, usually)
     * and may have a sector assignment.  The line segment may belont
     * to at most 2 sectors.  The first is passed in when created, the
     * second is set later by using setSectorListVal() method.
     * <p>
     * When created, the line segment will also determine if this is
     * a vertical line segement.  If so, it sets it's isVertical variable.
     *
     * @param n is an integer name for this line (usually an index into an array).
     * @param x_1 is the x value for the first point of line segment
     * @param y_1 is the y value for the first point of line segment
     * @param x_2 is the x value for the second point of line segment
     * @param y_2 is the y value for the second point of line segment
     * @param sec is the first sector to which this line segment belongs
     */
    public LineSegment(int n, double x_1, double y_1, double x_2, double y_2,
            int sec) {
        name = n;
    x1 = x_1;
    y1 = y_1;
    x2 = x_2;
    y2 = y_2;

    // find m
    if (!Util.isEquiv((x1 - x2), 0)) {
        m = (y1 - y2) / (x1 - x2);
        vertical = false;
    } else {
        m = 0;
        vertical = true;
    }

    // find b which is   y - mx
    b = y1 - (m * x1);

    sectorList = new int [2];
    sectorList[0] = sec;
    sectorList[1] = -1;
    }
    /**
     * This method will create a string displaying values of all
     * variables of this LineSegment.
     * @return the formatted string for System.out.println, for example
     */
    public String toString() {
    return new String("name: " + name +
            ", x1: " + x1 + ", y1: " + y1 +
            ", x2: " + x2 + ", y2: " + y2 +
            ",\n\t m: " + m + ", b: " + b +
            ", s[0]: " + sectorList[0] +
            ", s[1]: " + sectorList[1]);
    }

    public void setName(int n) {
        name = n;
    }
    public int getName() {
        return name;
    }

    public void setX1(double x) {
        x1 = x;
    }
    public double getX1() {
        return x1;
    }

    public void setY1(double y) {
        y1 = y;
    }
    public double getY1() {
        return y1;
    }

    public void setX2(double x) {
        x2 = x;
    }
    public double getX2() {
        return x2;
    }

    public void setY2(double y) {
        y2 = y;
    }
    public double getY2() {
        return y2;
    }

    public double getM() {
        return m;
    }
    public double getB() {
        return b;
    }
    public boolean isVertical() {
    return vertical;
    }

    /**
     * The sectorList keeps track of which sectors share this line
     * segment in the UTMOST2 model.
     * @param i is the index into the sector array - either 0 or 1
     * @param val is the value to which the array element is set.
     */
    public void setSectorListVal(int i, int val) {
        sectorList[i] = val;
    }
    /**
     * The sectorList keeps track of which sectors share this line
     * segment in the UTMOST2 model.  If the retrieved value is equal
     * to -1, this means that there is not a sector sharing this line.
     * The 0 element of the array should NOT be -1 if this line segment
     * is part of the sector description.
     * @param i is the index into the sector array - either 0 or 1
     * @return the value of the sector array at index i.
     */
    public int getSectorListVal(int i) {
        return sectorList[i];
    }

    public int [] getSecList() {
        return sectorList;
    }

    /**
     * This method will figure out if this line segment is equivalent
     * to the line segment defined by the passed in parameters.
     * <p>
     * This method must use the Util.isEquiv() method to see if the
     * points are the same, because all values are floating point!
     *
     * @see Util#isEquiv
     * @param tx1 is the x value for the first point of line segment
     * @param ty1 is the y value for the first point of line segment
     * @param tx2 is the x value for the second point of line segment
     * @param ty2 is the y value for the second point of line segment
     * @return true if the segments are equivalent, false otherwise.
     */
    public boolean isEquiv(double tx1, double ty1, double tx2, double ty2) {
    if (Util.isEquiv(tx1, x1) && Util.isEquiv(ty1, y1) &&
        Util.isEquiv(tx2, x2) && Util.isEquiv(ty2, y2)) {
        return true;
    } else if (Util.isEquiv(tx1, x2) && Util.isEquiv(ty1, y2) &&
               Util.isEquiv(tx2, x1) && Util.isEquiv(ty2, y1)) {
        return true;
    } else {
        return false;
    }
    }

    /**
     * This method determines if this line segement intersects with
     * the ln line segment.  If it does, it will set the Boundary
     * Cross (x,y) of the user to the intersection (x,y) point.
     * <p>
     * Uses mx + b = y = nx + c  where m and b are for this, n and c for ln
     * <p>
     * This will result in x = (c - b) / (m - n)
     * and use this x to find y = mx + b
     *
     * @return true if the lines intersect in their bounds, false,
     * otherwise.
     * @see User
     *
     */
    public boolean intersects(LineSegment ln, User u) {
    /*
     * Find x and y where this and ln intersect
     *
     * Use mx + b = y = nx + c  where m and b are for this, n and c for ln
     *
     * This will result in x = (c - b) / (m - n)
     * and use this x to find y = mx + b
     */
    boolean thisXisOK, lnXisOK, thisYisOK, lnYisOK;

    thisXisOK = lnXisOK = thisYisOK = lnYisOK = false;

    double x, y;
    // simple if neither line is vertical:
    if (!ln.isVertical() && !isVertical()) {
        x = (ln.getB() - getB()) / (getM() - ln.getM());

        y = (getM() * x) + getB();

    } else { // if either line or both are vertical... handle these
         // special cases
        if (ln.isVertical() && !isVertical()) {
            x = ln.getX1();
            y = (getM() * x) + getB(); // use this for mx+b=y
        } else if (!ln.isVertical() && isVertical()) {
            x = getX1();
            y = (ln.getM() * x) + ln.getB(); // use ln for mx+b=y
        } else {
        // case where both lines are vertical, so see if X's equiv.
            if (getX1() != ln.getX1()) {
                return false;
            } else {
                /*
                 * I think this is fine!  this just means the user
                 * is travelling on a line, therefore intersects the
                 * line at all points.  Use the Y of ln which is
                 * between this Y1 and Y2.
                 */
                y = getY1();
                x = getX1();
                if ((getY1() <= ln.getY1()) &&
                        (ln.getY1() <= getY2())) {
                    y = ln.getY1();
                } else if ((getY1() <= ln.getY2()) &&
                        (ln.getY2() <= getY2())) {
                    y = ln.getY2();
                } else if (Util.isEquiv(getY2(), ln.getY2())) {
                    y = ln.getY2();
                } else if (Util.isEquiv(ln.getY1(), getY1())) {
                    y = ln.getY1();
                }

                //System.err.println("This is an interesting boundary case" +
                //    " where user is travelling on a boundary line");
                //System.err.println("There may be problems with output " +
                //    "(x,y) boundary cross locations");
                // return false;
            }
        }
    }
/* This is too much detail even for debugging!
    if (DebugLevel.debugging(DebugLevel.SEGMENT_DEBUGGING)) {
        System.out.println("This: name:" + getName() +
            " (" + getX1() + "," + getY1() + "),(" +
            getX2() + "," + getY2() + ") m:" + getM() + " b:" +
            getB() + " vertical: " + isVertical());

        System.out.println("ln: name:" + ln.getName() +
            " (" + ln.getX1() + "," + ln.getY1() + "),(" +
            ln.getX2() + "," + ln.getY2() + ") m:" + ln.getM() +
            " b:" + ln.getB() + " vertical: " + ln.isVertical());
    }
*/
    // see if (x,y) is in the bounds of each line segment

    // First check this line segment:
    // X1 <= x <= X2  OR  X2 <= x <= X1
    if (((getX1() <= x) && (x <= getX2())) ||
        ((getX2() <= x) && (x <= getX1())) ||
        (Util.isEquiv(getX2(), x)) || (Util.isEquiv(x, getX1()))) {
        thisXisOK = true;
//        System.out.println("thisXisOK is " + thisXisOK);
    }
    // Y1 <= y <= Y2  OR  Y2 <= y <= Y1
    if (((getY1() <= y) && (y <= getY2())) ||
        ((getY2() <= y) && (y <= getY1())) ||
        (Util.isEquiv(getY2(), y)) || (Util.isEquiv(y, getY1()))) {
        thisYisOK = true;
//        System.out.println("thisYisOK is " + thisYisOK);
    }

    // Second check the ln line segment:
    // X1 <= x <= X2  OR  X2 <= x <= X1
    if (((ln.getX1() <= x) && (x <= ln.getX2())) ||
        ((ln.getX2() <= x) && (x <= ln.getX1())) ||
        (Util.isEquiv(ln.getX2(), x)) || (Util.isEquiv(x, ln.getX1()))) {
        lnXisOK = true;
//        System.out.println("lnXisOK is " + lnXisOK);
    }
    // Y1 <= y <= Y2  OR  Y2 <= y <= Y1
    if (((ln.getY1() <= y) && (y <= ln.getY2())) ||
        ((ln.getY2() <= y) && (y <= ln.getY1())) ||
        (Util.isEquiv(ln.getY2(), y)) || (Util.isEquiv(y, ln.getY1()))) {
        lnYisOK = true;
//        System.out.println("lnYisOK is " + lnYisOK);
    }

    // if it is, return true, else return false

    if (thisXisOK && thisYisOK && lnXisOK && lnYisOK) {
        u.setXBoundaryCross(x);
        u.setYBoundaryCross(y);
        return true;
    } else {
        return false;
    }
    }
    /**
     * This method will pick an X value which is somewhere on the line
     * segment.
     * @return the value of the randomly picked X value.
     */
    public double getRandomXPoint() {
        double d;
        double ret;
        if (x1 > x2) {
            d = x1 - x2;
            ret = ((Math.random() * 32768) % d) + x2;
            return ret;
    } else if (Util.isEquiv(x1, x2)) { // vertical line
            return x1;
        } else {
            d = x2 - x1;
            ret = ((Math.random() * 32768) % d) + x1;
            return ret;
        }
    }

    /**
     * This method will pick a Y value which is somewhere on the line
     * segment.
     * @return the value of the randomly picked Y value.
     */
    public double getRandomYPoint() {
        double d;
        double ret;
        if (y1 > y2) {
            d = y1 - y2;
            ret = ((Math.random() * 32768) % d) + y2;
            return ret;
    } else if (Util.isEquiv(y1, y2)) { // horizontal line
            return y1;
        } else {
            d = y2 - y1;
            ret = ((Math.random() * 32768) % d) + y1;
            return ret;
        }
    }

    /**
     * This method will calculate the Y value which is equal to
     * mx + b for this line segment.
     * @return the Y value.
     */
    public double calcYPoint(double x) {
        return (m * x + b);
    }
}
