/*
 * ModelSetup Panel
 *
 * Author: M. Heidi McClure
 * Date Started: 9/1/96
 * Date Last Revised: 4/12/97
 */

import java.awt.*;
import java.io.*; // for DataInputStream
import java.util.*; // for StringTokenizer

/**
 * This class is the top portion of UTMOST2.  It contains the Model
 * Setup parameter selections.  Except for the zoom entry field, when
 * any item in this panel changes, a NEW model is initiated.
 * <p>
 *
 * @author      M. Heidi McClure
 * @version UTMOST2 1.0
 * @see MapPanel
 * @see UTMOST2
 * @see UserStats
 * @see MapArea
 */
public class ModelSetup extends Panel {

//    private Label mPattern =        new Label("Pattern:");
    private Choice mPatternChoice =   new Choice();
    private Label mNumUsers =       new Label("Users:");
    private TextField mNumUsersText =   new TextField("25", 3);
    private Label mHexSize =    new Label("Zoom:");
    private TextField mHexSizeText =   new TextField("66", 3);
    private String mCircularModel =          new String("Model:");
    private Label mModel = new Label(mCircularModel);
    private Choice mCircularModelChoice =   new Choice();
    private String mLinearModel =          new String("Model:");
    private Choice mLinearModelChoice =   new Choice();
//    private Label mUserBehavior =          new Label("Behavior:");
    private Choice mUserBehaviorChoice =   new Choice();

	private Button moveMapButton = new Button("Move");
	private Label xoffsetLabel = new Label("x:0   ");
	private Label yoffsetLabel = new Label("y:0   ");

    private UTMOST2 the;

    private Sector [] sector;
    private LineSegment [] lineSeg;

    public ModelSetup() {
    }
    /**
     * Constructor which will create a ModelSetup object.
     *
     * @param u is the UTMOST2 object which called this constructor
     */
    public ModelSetup(UTMOST2 u) {
        the = u;
        setLayout(new FlowLayout(FlowLayout.CENTER, 3, 3));

//        add(mPattern);
		add(mModel);  // initially 7 Cell info shows...
        mPatternChoice.addItem("7 Cell");
        mPatternChoice.addItem("4 Cell");
        
		add(mPatternChoice);
        mCircularModelChoice.addItem("Random Arriving");
        mCircularModelChoice.addItem("Edge Arriving");
        mCircularModelChoice.addItem("Random Leaving");
        mCircularModelChoice.addItem("Center Leaving");
        mCircularModelChoice.addItem("Random");
        add(mCircularModelChoice);

        mLinearModelChoice.addItem("Random Arriving");
        mLinearModelChoice.addItem("Arriving to Right");
        mLinearModelChoice.addItem("Random Leaving");
        mLinearModelChoice.addItem("Leaving to Left");
        mLinearModelChoice.addItem("Random");

        add(mNumUsers);
        add(mNumUsersText);

//        add(mUserBehavior);
        mUserBehaviorChoice.addItem("Constant");
        mUserBehaviorChoice.addItem("Regenerate");
        add(mUserBehaviorChoice);

        add(mHexSize);
        add(mHexSizeText);

		add(moveMapButton);
		moveMapButton.resize(30,30);
		add(xoffsetLabel);
		add(yoffsetLabel);

    }

    public void setUTMOST2(UTMOST2 u) {
        the = u;
    }

	public void setOffsetText(int x,int y){
		xoffsetLabel.setText("x:" + x);
		yoffsetLabel.setText("y:" + y);
	}

    public void setSector(Sector [] s) {
        sector = s;
        the.sector = s;
    }

    /**
     * @return the line segment array for this model.
     */
    public LineSegment [] getLineSeg() {
        return lineSeg;
    }

    public void setLineSeg(LineSegment [] ln) {
        lineSeg = ln;
    }

    /**
     * @return the selected pattern (7 Cell or 4 Cell).
     */
    public String getPattern() {
        return mPatternChoice.getSelectedItem();
    }

	public void setPattern(String s){
		mPatternChoice.select(s);
	}

	/*
	return the number off cells in model
	*/
	public int getCells()
	{
		if(mPatternChoice.getSelectedIndex() == 0)
			return(7);
		else
			return(4);
	}

    /**
     * @return the selected model (Arriving, Leaving or Random).
     */
    public String getModel() {
        String s;
        if (mCircularModelChoice.isShowing()) {
            s = new String(mCircularModelChoice.getSelectedItem());
        } else {
            s = new String(mLinearModelChoice.getSelectedItem());
        }
        if (s.indexOf("Arriving") != -1) {
            return "Arriving";
        } else if (s.indexOf("Leaving") != -1) {
            return "Leaving";
        } else {
            return "Random";
        }
    }

	public void setModel(String s){
		if (mCircularModelChoice.isShowing()) {
            mCircularModelChoice.select(s);
        } else {
            mLinearModelChoice.select(s);
        }
	}


    /**
     * @return the selected start type (Random or Perimeter).  Note
     * that Perimeter includes starts from center, left, right or
     * perimeter edge of model.
     */
    public String getStartType() {
        String s;
        if (mCircularModelChoice.isShowing()) {
            s = new String(mCircularModelChoice.getSelectedItem());
        } else {
            s = new String(mLinearModelChoice.getSelectedItem());
        }
        if (s.indexOf("Random") != -1) {
            return "Random";
        } else {
            return "Perimeter"; // this includes center, left, right and edge
        }
    }

    /**
     * @return the selected user behavior (Constant or Regenerate).  Note
     * that Constant means a user is never reinitialized during a model's
     * run.  Regenerate means if a user travels outside of the model or
     * reaches its destination (center), it is reinitialized.
     */
    public String getBehavior() {
        String s;
        if (mUserBehaviorChoice.getSelectedItem().equals("Constant")) {
            return "Constant";
        } else {
            return "Regenerate";
        }
    }

	public void setBehavior(String s){
		mUserBehaviorChoice.select(s)	;
	}

    /**
     * @return the sector array for this model pattern.
     */
    public Sector [] getSectors() {
        return sector;
    }

	public void setNumUsers(String s){
		mNumUsersText.setText(s);
	}


    /**
     * action() method is for handling Button, TextField, Choice
     * and CheckBox action events ONLY. Geary, pg. 72 and 82-.
     */
    public boolean action(Event evt, Object arg) {
        int users;
        double zoom;

        users = new Integer(mNumUsersText.getText()).intValue();
        if (users > 5000) {
            the.asNeededPrintln("Max number of users is 5000...");
            mNumUsersText.setText("25");
            return true;
        }
        zoom = new Double(mHexSizeText.getText()).doubleValue();

        if (evt.target == mPatternChoice) {
            updateModelMenus();
            readSectors();
            setupLineSegments();
            the.sector = sector;
			the.selectedUser.startNew();
            the.map.updateMap(getModel(), getPattern(),
                                getBehavior(), getStartType(), users, zoom);
        }
        if (evt.target == mHexSizeText) {
            the.map.updateMap(zoom);
        }
        if (evt.target == mNumUsersText ||
            evt.target == mUserBehaviorChoice ||
            evt.target == mCircularModelChoice ||
            evt.target == mLinearModelChoice) {
			the.selectedUser.startNew();
            the.map.updateMap(getModel(), getPattern(),
                                getBehavior(), getStartType(), users, zoom);
        }
		if (evt.target == moveMapButton){
			the.map.changeMoveMap();
		}
        return true;
    }

    /**
     * This method is called when the pattern has changed.  This method
     * will display either the 7 Cell pattern or the 4 Cell pattern
     * popup menus.
     */
    public void updateModelMenus() {
        String curPattern = new String(getPattern());

        if (getPattern().equals("7 Cell")) {
            if (mLinearModelChoice.isShowing()) {
                mModel.setText(mCircularModel);
                hide();
                invalidate();
                removeAll();
                add(mModel);
//                add(mPattern);
                add(mPatternChoice);
                add(mCircularModelChoice);
                add(mNumUsers);
                add(mNumUsersText);
//                add(mUserBehavior);
                add(mUserBehaviorChoice);
                add(mHexSize);
                add(mHexSizeText);
				add(moveMapButton);
				moveMapButton.resize(30,30);
				add(xoffsetLabel);
				add(yoffsetLabel);
                show();
                getParent().validate();
            }
        } else { // "4 Cell"
            if (mCircularModelChoice.isShowing()) {
                mModel.setText(mLinearModel);
                hide();
                invalidate();
                removeAll();
                add(mModel);
//                add(mPattern);
                add(mPatternChoice);
                add(mLinearModelChoice);
                add(mNumUsers);
                add(mNumUsersText);
//                add(mUserBehavior);
                add(mUserBehaviorChoice);
                add(mHexSize);
                add(mHexSizeText);
				add(moveMapButton);
				moveMapButton.resize(30,30);
				add(xoffsetLabel);
				add(yoffsetLabel);
                show();
                getParent().validate();
            }
        }
    }
    /**
     * This method will read in from the correct sector_data file the
     * information needed for the sector array.  This method will
     * initialize or re-initialize the sector array of UTMOST2.
     * <p>
     * @see Sector
     */
    public void readSectors() {
        // read in from sector_data info about each sector in the
        // sectors array.
        String filename;
        // Check to see if 4 cell or 7 cell pattern.
        if (getPattern().equals("7 Cell")) {
                filename = "7_sector_data";
                the.MODEL_HEIGHT = the.RADIUS * 5;
                sector = new Sector [21];
        } else {
                filename = "4_sector_data";
                the.MODEL_HEIGHT = the.RADIUS * 2;
                sector = new Sector [12];//[99]; ///99??
        }
        // initialize Sector Array
        for (int i = 0; i < sector.length; i++) {
            sector[i] = new Sector();
        }
        try {
            DataInputStream istream = new DataInputStream(
                new FileInputStream(filename));
            String line;
            line = istream.readLine();
            line = line.trim();
            while (line.startsWith("=====") != true) {
                line = istream.readLine();
            }
            int c = -1;
            while ((line = istream.readLine()) != null) {
                StringTokenizer tmp;
                String cel;
                String sec;
                int s;
                int corner;
                double x;
                double y;
                double rad;
                double rng;
                double dir;
                tmp = new StringTokenizer(line);
                if (!tmp.hasMoreTokens()) {
                    if (DebugLevel.debugging(DebugLevel.SECTOR_DEBUGGING)) {
                        the.asNeededPrintln("No more Tokens, break");
                    }
                    break;
                }
                sec = new String(tmp.nextToken());      //sector name
                s = Integer.parseInt(sec);
                corner = s % 3 + 1;
                dir = Double.valueOf(tmp.nextToken()).doubleValue();
                x = Double.valueOf(tmp.nextToken()).doubleValue();
                y = Double.valueOf(tmp.nextToken()).doubleValue();
                rad = the.RADIUS;
                rng = the.RADIUS *
                Double.valueOf(tmp.nextToken()).doubleValue();

                sector[s] = new Sector(s, corner, x, y, dir, rad, rng);

                if (DebugLevel.debugging(DebugLevel.SECTOR_DEBUGGING)) {
                    the.asNeededPrintln("Sector[" + s + "]: " +
                    sector[s]);
                }
            }
        }catch (IOException e) {
                System.err.println("readSectors() exception");
                System.err.println(e);
                return;
        }
    }

    /**
     * This method will initialize or re-initialize the line segment
     * array.  The last element in the array is initialized with all
     * attributes equal to -1.
     * <pre>
     * For each segment add its lines to the line segment array
     *
     * The line segment array basically looks like:
     *
     * ln   x1 y1   x2 y2   m b     sector names (up to two.)
     * ==   == ==   == ==   = =     =========================
     * 0                            an array of two elements
     * 1
     * 2
     * 3
     * 4
     * etc.
     *
     * The sequence of lines should be in this order:
     *
     *               doub5
     *                   .
     *                 / | \
     *                /  |  \
     *           ln3 /   |   \
     *              /    |    \
     *             /     |     \ln6
     *            /      |      \
     *           /       |       \
     *      doub0.    ln2|       .doub4
     *           |       |       |
     *           |       |       |
     *           |       |       |
     *        ln0|       .       |ln5
     *           |     /   \     |
     *           | ln1/  C  \ln4 |   where C = Center
     *           |   /       \   |
     *           |  /         \  |
     *           | /           \ |
     *      doub1.               .doub3
     *            \             /
     *             \           /
     *              \         /
     *           ln7 \       / ln8
     *                \     /
     *                 \   /
     *                  \ /
     *                   .
     *               doub2
     *
     * doub refers to the doub_x and doub_y arrays which indicate the
     * (x,y) coordinates of each point.
     * </pre>
     * <p>
     * @see LineSegment
     * @see Sector
     * @see UTMOST2
     * @return the LineSegment array created by this method.
     */
    public LineSegment [] setupLineSegments() {
    int i, j, k;
        // setup line segment array based on current sector setup
        LineSegment [] ln;

        ln = new LineSegment [sector.length * 4]; // note this is overkill
                                        // many segments are shared!

        double centerX, centerY;

        double p_x[] = {0,0,0,0,0};
        double p_y[] = {0,0,0,0,0};


        double cur_x[] = {0,0,0,0,0,0,0};
        double cur_y[] = {0,0,0,0,0,0,0};

        double doub_x[] = {0,0,0,0,0,0,0};
        double doub_y[] = {0,0,0,0,0,0,0};

//        double radius = the.map.getRadius();
        double radius = the.RADIUS;

// X, Y and points are used to draw hexagon segments
        double X[] = {-0.866, -0.866, 0, 0.866, 0.866, 0, -0.866};
        double Y[] = {-0.500, 0.5, 1.0, 0.5, -0.500, -1.000, -0.500};

        int points = X.length;

        //setup initial x's and y's
        for (i = 0; i < points; i++) {
            cur_x[i] = X[i] * radius;
            cur_y[i] = Y[i] * radius;
        }

        k = 0; // to be used to index into LineSegment array ln
        for (i = 0; i < sector.length; i++) {
            if (sector[i].getName() == -1) {
                if (DebugLevel.debugging(DebugLevel.SECTOR_DEBUGGING)) {
                    the.asNeededPrintln("End of Segment array reached");
                }
                break;
            }
            centerX = sector[i].getBaseX();
            centerY = sector[i].getBaseY();
/* this is an annoying printout, so leave it out unless *really* debugging!
        if (DebugLevel.debugging(DebugLevel.SECTOR_DEBUGGING)) {
            the.asNeededPrintln("centerX: " + centerX +
                    " centerY: " + centerY);
            the.asNeededPrintln("(x,y)");
        }
*/
            //setup x's and y's based on centerX and Y and direction?
            for (j = 0; j < points; j++) {
                doub_x[j] = centerX + cur_x[j];
                doub_y[j] = centerY + cur_y[j];
/*this is an annoying printout, so leave it out unless *really* debugging!
        if (DebugLevel.debugging(DebugLevel.SECTOR_DEBUGGING)) {
            the.asNeededPrint("(" + doub_x[j] + "," + doub_y[j] + ")");
        }
*/
            }
/*this is an annoying printout, so leave it out unless *really* debugging!
        if (DebugLevel.debugging(DebugLevel.SECTOR_DEBUGGING)) {
            the.asNeededPrintln(" ");
        }
*/
        switch (i % 3) {
        case 0 :
        p_x[0] = doub_x[0];
        p_x[1] = doub_x[1];
        p_x[2] = centerX;
        p_x[3] = doub_x[5];
        p_x[4] = doub_x[0];

        p_y[0] = doub_y[0];
        p_y[1] = doub_y[1];
        p_y[2] = centerY;
        p_y[3] = doub_y[5];
        p_y[4] = doub_y[0];
        break;
        case 1 :
        p_x[0] = doub_x[5];
        p_x[1] = centerX;
        p_x[2] = doub_x[3];
        p_x[3] = doub_x[4];
        p_x[4] = doub_x[5];

        p_y[0] = doub_y[5];
        p_y[1] = centerY;
        p_y[2] = doub_y[3];
        p_y[3] = doub_y[4];
        p_y[4] = doub_y[5];
        break;
        case 2 :
        p_x[0] = centerX;
        p_x[1] = doub_x[1];
        p_x[2] = doub_x[2];
        p_x[3] = doub_x[3];
        p_x[4] = centerX;

        p_y[0] = centerY;
        p_y[1] = doub_y[1];
        p_y[2] = doub_y[2];
        p_y[3] = doub_y[3];
        p_y[4] = centerY;
        break;
        default :
        the.asNeededPrintln("an error in doing line segments");
        }
        // load LineSegments as needed... update existing...
        for (j = 0; j < 4; j++) {
        // check to see if line already exists
        int done = 0;
        for (int m = 0; m < k; m++) {
            if (ln[m].isEquiv(p_x[j], p_y[j], p_x[j+1], p_y[j+1])) {
            // add sector name to this ln[m]
            ln[m].setSectorListVal(1, sector[i].getName());
            done++;
            }
        }
        if (done == 0) { // add this to ln list at index k
            ln[k] = new
            LineSegment(k, p_x[j], p_y[j], p_x[j+1], p_y[j+1],
                sector[i].getName());
            k++;
        }
        }
        }

        // now set the last element in LineSegment array to be a
        // definite end value.
        ln[k] = new LineSegment(-1, -1, -1, -1, -1, -1);

    if (DebugLevel.debugging(DebugLevel.SEGMENT_DEBUGGING)) {
        the.asNeededPrintln("k: " + k + ", i: " + i );
        for (i = 0; i <= k; i++) {
            the.asNeededPrintln("ln[" + i + "]: " + ln[i].toString());
        }
    }
    setLineSeg(ln);
    return ln;
    }
	
	public void updateToLoaded(){
		int users = new Integer(mNumUsersText.getText()).intValue();
		double zoom = new Double(mHexSizeText.getText()).doubleValue();
		updateModelMenus();
        readSectors();
        setupLineSegments();
        the.sector = sector;
		the.map.updateMap(getModel(), getPattern(),
                      getBehavior(), getStartType(), users, zoom);
	}
}

