/*
 * Util class
 *
 * Author: M. Heidi McClure
 * Date Started: 2/18/97 with calcTheta...
 * Date Util class created: 4/5/97
 * Date Last Revised: 4/12/97
 */

import java.awt.*;

/**
 * This class contains a set of utility methods used by the UTMOST2
 * code.
 * <p>
 * @author      M. Heidi McClure
 * @version UTMOST2 1.0
 */
public final class Util {

    /**
     * calcTheta figures out the angle in degrees between the point
     * (x,y), the point (bx,by) and the point on the line segment s
     * which is not (bx,by).  That is, this method assumes that segment
     * s does contain (bx,by) as one of its endpoints.
     * <p>
     * @param x x of point 1
     * @param y y of point 1
     * @param bx x of point 2
     * @param by y of point 2
     * @param s line segment which contains (bx,by) as an end point and
     * its other end point is the point with which to make the theta
     * calculation.
     * @return the angle in degrees
     */
    public static double calcTheta(double x, double y, double bx, double by,
                                LineSegment s) {
        double a, b, c;
    double ret;
        if (DebugLevel.debugging(DebugLevel.FLOW_DEBUGGING)) {
            System.out.println(
                "inside calcTheta. x = " + x + " y = " + y + " bx = " + bx +
                " by = " + by + " segment: " + s);
        }

        // for each emminating segment, first figure out which point
        // of the segment is the (baseX, baseY).  Then find lengths
        // b and c
        a = b = c = 1;

        a = calcLength(x, y, bx, by);

        if (bx == s.getX1() && by == s.getY1()) {
            b = calcLength(s.getX2(), s.getY2(), bx, by);
            c = calcLength(s.getX2(), s.getY2(), x, y);
        } else {
            b = calcLength(s.getX1(), s.getY1(), bx, by);
            c = calcLength(s.getX1(), s.getY1(), x, y);
        }
        if (DebugLevel.debugging(DebugLevel.VISIBLE_DEBUGGING)) {
            System.out.println("     a: " + a + " b: " + b + " c: " + c);
        }
        // Now calculate C (the angle at (baseX, baseY) of this triangle
        if (isEquiv(a, 0) || isEquiv(b, 0)) {
            ret = 0;
        } else {
            ret = Math.acos(((a * a) + (b * b) - (c * c))/(2 * a * b));
    }
    return ret;
    }
    /**
     * calcTheta figures out the angle in degrees between the point
     * (ax,ay), the point (bx,by) and the point (cx,cy).
     * <p>
     * @param ax x of point a
     * @param ay y of point a
     * @param bx x of point b
     * @param by y of point b
     * @param cx x of point c
     * @param cy y of point c
     * @return the angle at point (bx,by)
     */
    public static double calcTheta(double ax, double ay, double bx, double by,
                                double cx, double cy) {
        double a, b, c;
        double ret;
//        System.out.println(
//                "inside calcTheta. ax = " + ax + " ay = " + ay + " bx = " + bx +
//                " by = " + by + " cx = " + cx + " cy = " + cy);

        a = b = c = 1;

        a = calcLength(cx, cy, bx, by);
        b = calcLength(ax, ay, bx, by);
        c = calcLength(ax, ay, cx, cy);
//        System.out.println("     a: " + a + " b: " + b + " c: " + c);
        // Now calculate C (the angle at (bx,by) of this triangle
        if (isEquiv(a, 0) || isEquiv(b, 0)) {
            ret = 0;
        } else {
            ret = Math.acos(((a * a) + (b * b) - (c * c))/(2 * a * b));
        }
        return (ret);
    }

    /**
     * this method returns the length between the two points passed in.
     * @param x1 x of first point
     * @param y1 y of first point
     * @param x2 x of second point
     * @param y2 y of second point
     * @return the distance between the two points.
     */
    public static double calcLength(double x1, double y1, double x2, double y2) {
        return Math.sqrt((x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1));
    }

    /**
     * this method returns true if the two floating point numbers are
     * basically the same value.  This method is needed to avoid
     * computer errors in calculating if one floating point is equal to
     * another!
     */
    public static boolean isEquiv(double x, double y) {
        if (x < y + 0.0001 && x > y - 0.0001) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * This method takes an (x,y) coordinate and a destination coordinate
     * (cx,cy) and figures out what that direction is.
     * <p>
     * <pre>
     * based on quadrant, use the following logic:
     *
     * always use dy/dx = tan(theta)
     *
     * use arriving center pattern to visualize this picture (7 cell Arriving):
     *              |
     *   moving     | moving
     *   270 to 330 | 180 to 270
     *   degrees    | degrees
     *              |
     *  ----------------------------
     *              |
     *   moving     | moving
     *   0 to 90    | 90 to 180
     *   degrees    | degrees
     *              |
     *              |
     *
     * for lower left,  use theta = atan(dy/dx)
     * for lower right, use theta = 180 - atan(dy/dx)
     * for upper right, use theta = atan(dy/dx) + 180
     * for upper left,  use theta = 360 - atan(dy/dx)
     * </pre>
     * @param x the starting x location
     * @param y the starting y location
     * @param cx the destination x location (center for 7 cell arriving)
     * @param cy the distination y location (center for 7 cell arriving)
     *
     * @return the direction in degrees
     */
    public static double getDirection(double x, double y,
            double cx, double cy, double rad2deg) {
    // set direction to be towards center which is (cx,cy)

    // use atan(x/y) converted to degrees.  1 rad = 57.29578 deg
    //                                     (1/57.29578)rad = 1 deg
        double dx;
        double dy;

        double theta;

        dx = Math.abs(cx - x);
        dy = Math.abs(cy - y);

		if (dy==0 && dx==0)
		{
			theta = 0;
		}
		else
		{
			if (cx > x) {
				if (cy > y) {   // upper left
					theta = (Math.atan(dy / dx)) * rad2deg;
					theta = 360 - theta;
				} else {        // lower left
					theta = (Math.atan(dy / dx)) * rad2deg;

				}
			} else {
				if (cy > y) {   // upper right
					theta = (Math.atan(dy / dx)) * rad2deg;
					theta = theta + 180;
				} else {        // lower right
					theta = (Math.atan(dy / dx)) * rad2deg;
					theta = 180 - theta;
				}
			}
		}
        return theta;
    }

    /**
     * This method originally was only in User.java... moved
     * here so it could be used in UserStats.java
     *
     * @param convert the integer name to convert to a string
     * @return the human readable sector name like 4C or 2B
     */
    public static String getSectorString(int convert) {
        int cell;
        int sect;
        StringBuffer ret = new StringBuffer();
        if (convert != -1) {
            cell = (1 + convert / 3) + (int)'0';
            sect = (convert % 3) + (int)'A';
            ret.append((char)cell);
            ret.append((char)sect);
            return(ret.toString());
        } else {
            return("Out");
        }
    }

    public static int getSectorInt(String s) {
		if(s!="Out")
		{
        int ret;
        int c0 = (int)s.charAt(0) - (int)'0' - 1; // will be multiplier
        int c1 = (int)s.charAt(1) - (int)'A'; // should be 0, 1 or 2
        ret = (c0 * 3) + c1;
        return ret;
		}
		else
			return (-1);
    }



}

