/**
 * UTMOST2 main application
 * <p>
 * This is the main driving program for the UTMOST
 * User Traffic MOdeling and Simulation Tool.
 * This version incorporates improved algorithms for boundary cross
 * calculations and for visible sector calculations
 * <p>
 * Author: M. Heidi McClure
 * Date Started: 9/1/96
 * Date Last Revised: 4/20/97
 * <p>
 * Note: this will require JDK1.1 in order to allow for printing!!!
 * Until JDK1.1 is available, this stuff will be commented out...
 * JDK1.1 also brings different handling of changing cursor from
 * arrow to "wait" icons. (pg 430 of Just Java, 2nd ed. by Peter
 * van der Linden)
 * <p>
 * <p>
 * This class also controls the creation of all of the panels which
 * provide the graphical user interface.  It also sets up the users,
 * sectors and other needed objects for the modeling.
 * <p>
 * @author      M. Heidi McClure
 * @version UTMOST 1.0
 * @see MapPanel
 * @see UTMOST
 * @see ModelSetup
 * @see MapArea
 */

import java.awt.*;

public class UTMOST2 extends Frame {

    /**
     * The time between each step in the model (10 seconds default)
     */
    public int TIME_INCREMENT = 10;
    /**
     * The radius of each cell (5km default)
     */
    public double RADIUS = 5.0;
    /**
     * The dot size of each dot representing a user (10 pixel default)
     */
    public int DOT_SIZE = 10;
    /**
     * The directory name (date part, only) of Simulator path (POCAT or RACESIM)
     */
    public String SIM_DIR_NAME = new String();
    /**
     * The model height (will be 5 * RADIUS for 7 Cell, 2 * RADIUS for 4 Cell)
     */
    public double MODEL_HEIGHT = RADIUS * 5;

    /**
     * The radians to degrees conversion (57.29578 default)
     */
    public double RAD_TO_DEG = 57.29578;

    /**
     * The maximum power displayed in the legend
     */
    public double POWER_MAX = 1.2;

    /**
     * The file separator based on the JVM ('/' in UNIX)
     */
    public String FILE_SEP;
    /**
     * The line separator based on the JVM ('\n' in UNIX)
     */
    public String LINE_SEP;
    /**
     * The path separator based on the JVM (':' in UNIX)
     */
    public String PATH_SEP;

	public int PowerControlType=1; /* Use POCAT as Default */ 
	public boolean generateBoundary=true; 
	private double SectorOverlap=15; //overlap in degrees
	
	POCATSettings PowerControlSetting = new POCATSettings();

	public boolean drawImage = false; //No image 
	public String ImageName = new String("test.gif");
	public int imageX = 0;
	public int imageY = 0;

    // MENUS
    Menu fileMenu;
    Menu editMenu;
    Menu viewMenu;
    Menu helpMenu;
    //}}

    // Model Attributes and Variables.
    User [] users;
    Sector [] sector;

    Console console;
    PropertySettings properties;
    MapArea map;
    MapPanel mapPanel;
    Legend legend;
    UserStats selectedUser;
    ModelSetup model;
	SectorInfo sectorInfo;
	LoadDialog load;

    /**
     * Constructor which will create a UTMOST2 object.  It will initialize
     * and fire up all of the panels on the UTMOST2 Frame.
     */
    public UTMOST2() {

        super("RACEWIN - Resource Allocation and admission Control Evaluator V1.0");

        setBackground(Color.lightGray);

        DebugLevel.parseDebugProperties();
        POWER_MAX = DebugLevel.getMaxWatts();

        FILE_SEP = System.getProperty("file.separator"); // / in Unix
        LINE_SEP = System.getProperty("line.separator"); // \n in Unix
        PATH_SEP = System.getProperty("path.separator"); // : in Unix

        // INITIALIZE MENUS
        MenuBar mb = new MenuBar();
        fileMenu = new Menu("File");
        fileMenu.add(new MenuItem("New"));
        fileMenu.add(new MenuItem("Open..."));
        fileMenu.addSeparator();
        fileMenu.add(new MenuItem("Print"));
        fileMenu.addSeparator();
        fileMenu.add(new MenuItem("Save"));
        fileMenu.add(new MenuItem("Save As..."));
        fileMenu.addSeparator();
        fileMenu.add(new MenuItem("Exit"));
        mb.add(fileMenu);
        editMenu = new Menu("Edit");
        editMenu.add(new MenuItem("Undo"));
        editMenu.addSeparator();
        editMenu.add(new MenuItem("Cut"));
        editMenu.add(new MenuItem("Copy"));
        editMenu.add(new MenuItem("Paste"));
        editMenu.addSeparator();
        editMenu.add(new MenuItem("Properties..."));
        mb.add(editMenu);
        viewMenu = new Menu("View");
		viewMenu.add(new MenuItem("Show SectorInfo"));
        viewMenu.add(new MenuItem("Show Console"));
        mb.add(viewMenu);
        helpMenu = new Menu("Help");
        helpMenu.add(new MenuItem("About..."));
        mb.add(helpMenu);
        setMenuBar(mb);

    //SETUP PANELS (initialize User array)

        console = new Console(this, "Console Window");
        console.hide();

        properties = new PropertySettings(this);
        properties.hide();

		load = new LoadDialog(this);
		load.hide();
		

	    users = new User[5000];
        map = new MapArea(this);
        model = new ModelSetup(this);
        sector = readSectors(model, map);
        model.setSector(sector);
        model.setupLineSegments();

        selectedUser = new UserStats(this);

        setInitUsers(map, model);  // must be called after curModel
                    // has been initialized (done above)

        setLayout(new BorderLayout(150,0));
        addNotify();

        EtchBox modelSettings = new EtchBox(model,
            "Current Model Settings");

        legend = new Legend(this);

        mapPanel = new MapPanel(this);

        add("North", modelSettings);
        add("South", selectedUser);
        add("Center", mapPanel);

		sectorInfo = new SectorInfo(this);
		sectorInfo.hide();

        resize( insets().left + insets().right + 742,
                insets().top + insets().bottom + 600);//564

        validate();
        show();
    }

    public synchronized void show() {
        move(10, 10);
        super.show();
    }

    /**
     * handleEvent is for handling mouse, keystroke, Window, ScrollBar,
     * List and some action events.  Geary, pg. 69
     */
    public boolean handleEvent(Event event) {

        if (event.id == Event.WINDOW_DESTROY) {
            hide();         // hide the Frame
            dispose();      // tell windowing system to free resources
            System.exit(0); // exit
            return true;
        }
        return super.handleEvent(event);
    }

    /**
     * action() method is for handling Button, TextField, Choice
     * and CheckBox action events ONLY. Geary, pg. 72 and 82-.
     */
    public boolean action(Event event, Object arg) {
        if (event.target instanceof MenuItem) {
            String label = (String) arg;
            if (label.equalsIgnoreCase("About...")) {
                selectedAbout();
                return true;
            } else if (label.equalsIgnoreCase("Print")) {
                selectedPrint();
                return true;
            } else if (label.equalsIgnoreCase("Exit")) {
                selectedExit();
                return true;
            } else if (label.equalsIgnoreCase("Open...")) {
                selectedOpen();
                return true;
            } else if (label.equalsIgnoreCase("Properties...")) {
                selectedProperties();
                return true;
            } else if (label.equalsIgnoreCase("Show Console")) {
                selectedConsole();
                return true;
            }else if (label.equalsIgnoreCase("Show SectorInfo")) {
                selectedSectorInfo();
                return true;
			}
        }

        return super.action(event, arg);
    }

    public static void main(String args[]) {
        new UTMOST2();
    }

    public void selectedOpen() {
        if (!load.isVisible()) {
			load.show();
		}
    }

    public void selectedProperties() {
        if (!properties.isVisible()) {
            properties.show();
        }
    }

	public void selectedSectorInfo(){
        if (!sectorInfo.isVisible()) {
			sectorInfo.calcSectorInfo();
            sectorInfo.show();
        }
    }

    public void selectedConsole() {
        if (console == null) {
            console = new Console(this, "UTMOST2 Console Window");
        }
    if (!console.isVisible()) {
            console.show();
    }
    }

    /**
     * Printing is disabled at this time.  Need to compile with JDK1.1
     * for printing (and of course, uncomment this section!)
     */
    public void selectedPrint() {
/*
        PrintJob pjob = getToolkit().getPrintJob(this,
        "Printing Test", (Properties)null);
        if (pjob != null) {
            Graphics pg = pjob.getGraphics();
            if (pg != null) {
                printAll(pg);
                pg.dispose(); // flush page
            }
            pjob.end();
        }
*/
    }
    public void selectedExit() {
        QuitBox theQuitBox;
        theQuitBox = new QuitBox(this);
        theQuitBox.show();
    }
    public void selectedAbout() {
        AboutBox theAboutBox;
        theAboutBox = new AboutBox(this);
        theAboutBox.show();
    }
    public void setInitUsers(MapArea map, ModelSetup currentModel) {
        // to read from user_init_data to get initial loc's for users
        // see commented out stuff, below...
        for (int i = 0; i < 5000; i++) {
            users[i] = new User(this, i);
        }
        for (int i = 0; i < 25; i++) { // initialize only 25 users...
            users[i].setInitUser(currentModel.getModel(),
                currentModel.getPattern(), currentModel.getStartType(),
                currentModel.getLineSeg());
        }


/* use below to read initial data from user_init_data file
        try {
            DataInputStream istream = new DataInputStream(
                new FileInputStream("user_init_data"));
            String line;
            int u;
            int x;
            int y;
            double coord;
            line = istream.readLine();
            line = line.trim();
            while (line.startsWith("=====") != true) {
                line = istream.readLine();
            }
            while ((line = istream.readLine()) != null) {
                StringTokenizer tmp = new StringTokenizer(line);
                if (!tmp.hasMoreTokens()) {
                    break;
                }
                u = Integer.parseInt(tmp.nextToken());
                System.out.print("user name is: " + u);
                users[u].setName(u);

                coord = (Double.valueOf(tmp.nextToken())).doubleValue();
                System.out.print(" X coord is: " + coord);
                users[u].setXLocation(coord);

                coord = (Double.valueOf(tmp.nextToken())).doubleValue();
                System.out.println(" Y coord is: " + coord);
                users[u].setYLocation(coord);
            }
        }catch (IOException e) {
                System.err.println(e);
                return;
        }
*/
    }
    public Sector [] readSectors(ModelSetup currentModel, MapArea map) {
        // read in from sector_data info about each sector in the
        // sectors array.
        currentModel.readSectors();
        return currentModel.getSectors();
    }

    public void asNeededPrint(String s) {
        System.out.print(s);
/*
        if (log != null) {
            log.print(s);
        }
*/
        if (console != null) {
            console.print(s);
        }
    }

    public void asNeededPrintln(String s) {
        System.out.println(s);
/*
        if (log != null) {
            log.println(s);
        }
*/
        if (console != null) {
            console.println(s);
        }
    }

	public double getSectorOverlap(){
		return SectorOverlap;
	}

	public void setSectorOverlap(double s){
		SectorOverlap=s;
	}

}
